C ----------------------------------------------------------------
C      PROGRAM ATMOD
C
C     VERSION 2.0   JUNE 1984
C     ALGORITHM DESIGNED BY DR. P.K. BHARTIA ; CODED BY DAVID LEE ,SASC
C	MODIFIED BY A. VASILKOV FOR USE IN TOMRAD_GAUSS-SEIDEL SIMULATIONS
C	FIXED A PROBLEM WITH AN OZONE VALUE AT THE LAST LAYER
C	AUGUST 17, 2000
C	Modified Oct. 25, 2000 to exclude reading an input file
C
C      PURPOSE -
C
C          FIND THE CUMULATIVE OZONE AMOUNT AND OZONE-WEIGHTED AVERAGE
C          TEMPERATURES AT 101 LEVELS FROM INPUT VALUES FOR 10 UMKEHR
C          LAYERS USING A SPLINE FIT.
C
C
C      METHOD / PROCEDURES
C
C          1) DEFINE INPUT AND OUTPUT PRESSURE LEVELS
C          2) COMPUTE ACCUMULATED OZONE AMOUNTS AT INPUT LEVELS
C          3) COMPUTE OZONE WEIGHTED AVERAGE TEMPERATURES ABOVE INPUT
C             PRESSURE LEVELS
C          4) PERFORM A SPLINE FIT TO TEMPERATURES AND ACCUMULATED
C             OZONES AT INPUT LEVELS
C          5) EVALUATE SPLINE FIT AT THE LOWEST 61 PRESSURE LEVELS
C          6) FIND SLOPE FOR THE ACCUMULATED OZONE AT THE 41ST. LEVEL
C          7) EVALUAATE ACCUMULATED OZONE AMOUNTS FOR THE TOP 40 LEVELS
C             FROM THE SLOPE COMPUTED FROM THE 41ST LEVEL
C          8) COMPUTE PRESSURES FOR ALTITUDES AT 1 KM INTERVALS
C
C
C     CALLING SEQUENCE - CALL RELAYR(DELX,XTOP,TMP10,TMPTOP,X101,TMP101,
C                        H,PS)
C
C
C     VARIABLE     TYPE    I/O   DESCRIPTION
C     --------     ----    ---   -----------
C
C     DELX(10)      R*8     I    LAYER OZONE AMOUNT (D.U) FOR
C                                10 UMKEHR LAYERS. (LAYERS 0-9, LAYER
C                                0 BEING THE BOTTOM HALF OF THE ATMOS-
C                                PHERE). INDEXING OF THE LAYERS ARE
C                                DESCRIBED IN THE TABLE BELOW
C
C     XTOP          R*8     I    CUMULATIVE OZONE AMOUNT(M-ATM-CM)
C                                ABOVE UMKEHR LAYER 9
C
C     P11(11)       R*8          PRESSURE LEVELS (ATM.) DEFINING THE 10
C                                UMKEHR LAYERS. INDEXING OF THE PRESSURE
C                                LEVELS ARE DESCRIBED IN THE TABLE
C                                BELOW
C
C     X11(11)       R*8          CUMULATIVE OZONE AMOUNT (M-ATM-CM)
C                                AT PRESSURE LEVELS DESCRIBED ABOVE
C                                FOR P11
C
C     PNOT          R*8    I     PRESSURE OF REFLECTING SURFACE
C
C     XPNOT         R*8    O     CUMULATIVE OZONE AT PNOT
C
C     P101(101)     R*8          PRESSURE (ATM.) AT 101 ATMOSPHERIC
C                                LEVELS THAT DEFINE THE OUTPUT LAYER
C                                OZONE AMOUNTS AND TEMPERATURES
C
C     X101(101)     R*8    O     OUTPUT CUMULATIVE OZONE AMOUNTS
C                                (ATM-CM) AT LEVELS CORRESPONDING
C                                TO P101.
C
C
C     Z11(11)       R*8          ALTITUDE (KM) CORRESPONDING TO P11
C                                DEFINING THE UMKEHR LAYERS
C
C     H(82)         R*8    O     ALTITUDE ARRAY AT 1 KM INTERVAL FROM
C                                GROUND TO 82 KM
C
C     PS(82)        R*8    O     PRESSURES CORRESPONDING TO THE HEIGHT
C                                AT THE H ARRAY
C
C
C-----------------------------------------------------------------------
C
C-----------------------------------------------------------------------
C
C     THE TABLE BELOW DESCRIBES THE ORDER OF INDICES FOR VARIABLES
C     IN THIS SUBROUTINE
C
C     PRESSURE       UMKEHR        INDICES         INDICES      INDEX
C      (ATM.)        LAYER #     FOR P11,X11       FOR DELX      FOR
C                                  & T11AVE        & TMP10       Z11
C
C      1/1024 -------------------------1--------------------------11
C                       9                             1
C      1/512---------------------------2--------------------------10
C                       8                             2
C      1/256---------------------------3---------------------------9
C                       7                             3
C      1/128---------------------------4---------------------------8
C                       6                             4
C      1/64 ---------------------------5---------------------------7
C                       5                             5
C      1/32 ---------------------------6---------------------------6
C                       4                             6
C      1/16 ---------------------------7---------------------------5
C                       3                             7
C      1/8  ---------------------------8---------------------------4
C                       2                             8
C      1/4  ---------------------------9---------------------------3
C                       1                             9
C      1/2  --------------------------10---------------------------2
C                       0                            10
C       1   --------------------------11---------------------------1
C
C
C--------------------------------------------------------------------
      subroutine profiles(itype,pfct,oz,temp)
      IMPLICIT INTEGER*4(I-N),REAL*8(A-H,O-Z)
      DIMENSION P11(11),P11LOG(11),P101(101),P101LG(101),P101LGI(101),
     1 X11(11),X11LOG(11),X101(101),X101LG(101),DELX(10),XLAY(101)
      DIMENSION Z11(11),PLGINV(11),P82(82),H(82),TMP10(10),TMP101(101),
     1 T11AVE(11),T11(11),H101(101),X82(82),X82LG(82),P82LG(82),        
     2 T101(101),H101INV(101),PLAY(101),T82(82),T82I(82),P82LGI(82),
     3 hinv(82),x83i(83),P83lgi(83),hs(100),ozone(100),ps(100),ts(100)
      DIMENSION PABLG(50),PBCLG(50),PCDLG(50),TMPAB(50),
     1           TMPBC(50),TMPCD(5)
      dimension delxinv(10),tmpinv(10)
      dimension h70(100),c70(100),p70inv(100),p101inv(101),x70inv(100),
     1          x101inv(101),t70inv(100),tmp101i(101),ozo70(100) 
      DIMENSION CX(11),BPARX(4),CT(11),C101(101),CZ(11),BPARZ(4),c82(82)
      dimension oz(11),temp(11)
C      character*15 fname       
C        open(11,file=fname,status='old')

        open(13,file='profile.dat',status='unknown')
c       print*,'When type is 0, this code generates a profile with the'
c       print*,'same  vertical resolution as the TOMRAD code (~ 1km)' 
c       print*,'When type is 1, it generates a profile with resolution:'
c       print*,'0 to 40 km in 0.5 km steps, 41 to 48 km in 1km steps,' 
c       print*,'50 to 70 km in 2 km steps'
c       print*,'When type is 2, it generates a profile with resolution:'
c       print*,'0 to 10 km in .2 km steps, 10.5 to 20 km in .5 km steps,' 
c       print*,'21 to 30 km in 1 km steps, 32 to 68 in 2km steps.'
c       print*,'Enter type of profile wanted (0,1 or 2, default is 1)'
c       read(5,*)itype
c       if(itype.gt.2)itype=1
c       
C  
C*****DEFINE PRESURES (ATM.) FOR LAYER OZONE AMOUNTS AND TEMPERATURES.
C     P11(11) IS THE AT THE BOTTOM OF THE ATMOSPHERE
C
      pnot = 1.00
      tmp0 = 273.0
c      print*,'Enter Pressure at the bottom of the atmosphere (in mb)'
c      read(5,*)pfct
      if(pnot.gt.1.00)pfct=1013.25
      DO 10 I=1,11
         P11(I)=1.0*2.**(I-11.)
	 write(6,*)p11(i)
         P11LOG(I)=DLOG(P11(I))
10    CONTINUE
      P11(11) = PNOT
      P11LOG(11) = DLOG(P11(11))
C
C*****DEFINE PRESSURES (ATM.) AT 101 LEVELS FOR OUTPUT OZONE AND
C     TEMPERATURES  P101(101) IS AT THE BOTTOM OF THE ATMOSPHERE
C
      DO 20 I=1,101
         P101(I)=PNOT*10.**((I-101.)/20.)
         P101LG(I)=DLOG(P101(I))
         p101inv(102-i) = p101(i)
20    CONTINUE
C
C*****COMPUTE CUMULATIVE OZONE AT 11 PRESSURE LEVELS & CONVERT TO ATM-CM
C
      do 9000 jpf = 1,1
C      read(11,*)(delxinv(i),i=1,10),xtop
C      read(11,*)(tmpinv(i),i=1,10),tmptop

      do i=1,10
        delxinv(i)=oz(i)
        tmpinv(i)=temp(i)
      enddo
      xtop=oz(11)
      tmptop=temp(11)

      totozn = xtop/1000.
      do 105 inv = 1,10
         jnv = 11 - inv
         delx(jnv) = delxinv(inv)
         tmp10(jnv) = tmpinv(inv)
         totozn = totozn + delxinv(inv)/1000. 
 105  continue
      X11(1)=XTOP/1000.
      X11LOG(1)=DLOG(X11(1))
      DO 110 I=2,11
         X11(I)=X11(I-1)+DELX(I-1)/1000.
         X11LOG(I)=DLOG(X11(I))
 110  CONTINUE
C
C****COMPUTE OZONE WEIGHTED AVERAGE TEMPERATURES AT 11 PRESSURE LEVEL
C
      TSUM = TMPTOP*XTOP/1000.
      T11AVE(1) = TMPTOP
      T11(1) = TMPTOP
      DO 150 I = 2,11
      TSUM = TSUM + TMP10(I-1)*DELX(I-1)/1000.
      T11AVE(I) = TSUM/X11(I)
      T11(I) = TMP10(I-1)
 150  CONTINUE 
C
C*****SET UP BOUNDARY VALUE PARAMETES FOR CUMULATIVE OZONE SPLINE FIT
C
C
C*****F'(NX)=1.707*LAYER 0 OZONE / TOTAL OZONE
C
      DELTAX=X11LOG(11)-X11LOG(10)
      DELTAP=P11LOG(11)-P11LOG(10)
      FNX=1.707*DELX(10)/1000./X11(11)
      BPARX(4)=6.0/DELTAP*(FNX-DELTAX/DELTAP)
C
C*****EVALUATE CUMULATIVE OZONE AT THE LOWEST 61 OUTPUT PREURE LEVEL
C     USING A SPLINE FIT TO POINT AT INPUT PRESSURE LEVELS
C
      NX=11
      IC=10
      L1=41
      L2=101
      M=L2-L1+1
      CALL SPLINE(P11LOG,X11LOG,NX,1.D30,FNX,CX)
      DO 175 I = L1,L2
         CALL SPLINT(P11LOG,X11LOG,CX,NX,P101LG(I),X101LG(I))
 175  CONTINUE
C
C***** EVALUATE CUMULATIVE OZONE AT PNOT
C
      XPNOT=X11(11)
      IF (PNOT.GE.P11(11)) GO TO 180
      P0LOG=DLOG(PNOT)
      CALL SPLINT(P11LOG,X11LOG,CX,NX,P0LOG,XP0LOG)
      XPNOT=DEXP(XP0LOG)
  180 CONTINUE
C
C*****EVALUATE CUMULTIVE OZONE AT UPPERMOST 40 LEVELS BASED ON
C     SLOPE COMPUTED FROM THE 41ST AND 42ND LEVELS
C
      SLOPE=(X101LG(L1)-X101LG(L1+1))/(P101LG(L1)-P101LG(L1+1))
C
      L1M1=L1-1
      DO 200 L=1,L1M1
      X101LG(L)=X101LG(L1)+SLOPE*(P101LG(L)-P101LG(L1))
  200 CONTINUE
C
C***** CONVERT LOG OF CUMULATIVE OZONE TO CUMULATIVE OZONE
C
      DO 250 L=1,101
      X101(L)=DEXP(X101LG(L))
      x101inv(102-l) = x101(l)
c      write(6,*)x101(l)
  250 CONTINUE
C
C*****EVALUATE OZONE-WEIGHTED AVERAGE TEMPARATURES FOR ATMOSPHERES
C     ABOVE LOWEST 61 OUTPUT PRESSURE LEVELS USING A SPLINE FIT TO 
C     POINTS AT INPUT PRESSURE LEVELS
C
      CALL SPLINE(P11LOG,T11AVE,NX,1.D30,1.D30,CT)
      DO 260 K = L1,L2
      CALL SPLINT(P11LOG,T11AVE,CT,NX,P101LG(K),TMP101(K))
 260  CONTINUE
C
C**** EVALUATION OF UNWEIGHTED TEMPERATURE PROFILE
C
      CALL SPLINE(P11LOG,T11,NX,1.D30,1.D30,CT)
      DO 270 K = L1,L2
      CALL SPLINT(P11LOG,T11,CT,NX,P101LG(K),T101(K))
 270  CONTINUE
C
C**** EVALUATE WEIGHTED AND UNWEIGHTED TEMPERATURES IN THE UPPER MOST 40
C     LAYERS BASED ON CONSTANT LAPSE RATE OF -1.5 DEGREES PER LAYER.
C  
      DO 300 L = 1,L1M1
      TMP101(L) = TMP101(L1) - 1.5*(L1-L)
      T101(L) = T101(L1) - 1.5*(L1-L)
 300  CONTINUE
C
C*****COMPUTE HEIGHTS Z11 (KM.) CORRESPONDING TO THE 11 INPUT PRESSURE
C     LEVELS P11 (IN INVERSE ORDER) USING THE EQUATIONS :
C     DZ/DLOGP=N*R*T/G
C     G=G0*(R0/(R0+Z))**2
C
C     CONSTANT IS G0/(N*R); Z11(1) IS AT BOTTOM OF ATMOSPHERE
C
      CONST=1.0/(-34.163)
      R0=6356.8
      DZHALF=2.5
      Z11(1)=0.
      PLGINV(1)=P11LOG(11)
      DO 400 I=2,11
         J=12-I
         PLGINV(I)=P11LOG(J)
         DLOGP=PLGINV(I)-PLGINV(I-1)
         Z=Z11(I-1)+DZHALF
         T=TMP10(J)
         DZ=CONST*((R0+Z)/R0)**2*T*DLOGP
         Z11(I)=Z11(I-1)+DZ
         DZHALF=DZ/2.0
400   CONTINUE
C
C*****PERFORM A SPLINE FIT TO LOG(P) AND ZC
C  
      CALL SPLINE(Z11,PLGINV,NX,1.D30,1.D30,CZ)
C
C*****SET UP ALTITUDE ARRAYS AT 1 KM CONSTANT INTERVALS
C
      DO 320 I=1,82
         H(I)=FLOAT(I-1)*1.0
         hinv(i) = 81. - h(i)
 320  CONTINUE
C
C*****EVALUATE LOG OF PRESSURE AT 1 KM INTERVALS FROM THE GROUND TO
C     THE TOP OF UMKEHR LAYER 9 BASED ON SPLINE FIT
C
      M=Z11(11)+1.
      DO 345 I = 1,M
         CALL SPLINT(Z11,PLGINV,CZ,NX,H(I),P82LGI(I))
345   CONTINUE
C
C*****EVALUATE LOG OF PRESSURES ABOVE UMKEHR LAYER 9 BASED ON CONSTANT
C     SCALE HEIGHT
C
      SLOPE=(P82LGI(M)-P82LGI(M-1))/(H(M)-H(M-1))
      MP1=M+1
      DO 450 I=MP1,82
      P82LGI(I)=P82LGI(M)+SLOPE*(H(I)-H(M))
 450  CONTINUE
C
c..
c.. Obtain altitude of 101 levels splining from the 82 level structure
c..
      do 454 i = 1,82
          K = 83 - I
          p83lgi(i) = p82lgi(i)
          P82LG(K) = P82LGI(I)
          P82(K)=DEXP(P82LGI(I))
 454  continue
      call spline(p82lg,h,82,1.d30,1.d30,c82)
      do 455 ih = 1,101
         call splint(p82lg,hinv,c82,82,p101lg(ih),h101inv(ih))
         h101(102-ih) = h101inv(ih)
         tmp101i(102-ih) = tmp101(ih) 
 455  continue
       conv = 4.66968e+04
c       pfct = 1013.25
       do 880 il = 1,99
          ps(il) = p101(102-il)*pfct
          hs(il) = h101inv(102-il)
          ts(il) = tmp101(102-il)
          ozone(il)=(x101(102-il)-x101(101-il))/
     1              (h101inv(101-il)-h101inv(102-il))
c          write(6,*)x101(102-il)-x101(101-il),
c     1	  h101inv(101-il)-h101inv(102-il)
 880   continue
       ps(100) = p101(2)*pfct
       hs(100) = h101inv(2)
       ts(100) = tmp101(2)
       ozone(100)= x101(2)/(h101inv(1) - h101inv(2))
       sum = 0
       do 882 il = 2,100
          sum = sum + 0.5*(hs(il)-hs(il-1))*(ozone(il)+ozone(il-1))
 882   continue
       con = totozn/sum
       if(itype.ne.0) go to 9000
       do 885 ik = 1,100
          write(13,1010)hs(ik),ps(ik),ozone(ik)*con/conv,ts(ik)
          write(6,1010)hs(ik),ps(ik),ozone(ik)*con/conv,ts(ik)
 885   continue
       go to 9005         
  890  CONTINUE
 9000  continue
c
c***** Generate high resolution profile
c
      do 324 i = 1,100
         if(itype.eq.1)then
           if(i.le.81) h70(i) = 0.5*float(i)-0.5
           if(i.ge.82.and.i.le.89)h70(i) = h70(i-1) + 1.0
           if(i.ge.90)h70(i) = h70(i-1) + 2.0
         else 
           if(i.le.51) h70(i) = 0.2*float(i)-0.2
           if(i.ge.52.and.i.le.71)h70(i) = h70(i-1) + 0.5
           if(i.ge.72.and.i.le.81)h70(i) = h70(i-1) + 1.0
           if(i.ge.82)h70(i) = h70(i-1)+ 2.0
         endif  
 324  continue
      call spline(h101,p101inv,100,1.d30,1.d30,c70)
      do 326 j = 1,100
         call splint(h101,p101inv,c70,100,h70(j),p70inv(j))
         p70inv(j) = p70inv(j)*pfct
 326  continue
      call spline(h101,x101inv,100,1.d30,1.d30,c70)
      do 328 j = 1,100
         call splint(h101,x101inv,c70,100,h70(j),x70inv(j))
 328  continue
      call spline(h101,tmp101i,100,1.d30,1.d30,c70)
      do 330 j = 1,100
         call splint(h101,tmp101i,c70,100,h70(j),t70inv(j))
 330  continue
       do 332 il = 1,99
          ozo70(il)=(x70inv(il)-x70inv(il+1))/(h70(il+1)-h70(il))
 332   continue
       ozo70(100)= x70inv(100)/(h70(100) - h70(99))
       sum = 0
       do 334 il = 2,100
          sum = sum + 0.5*(h70(il)-h70(il-1))*(ozo70(il)+ozo70(il-1))
 334   continue
       con = totozn/sum
       write(6,*)totozn,sum,con
       
C----------ozone at the last layer set to be a half of a prevoius
C----------value to fix a mistake
       
       	ozo70(100)=ozo70(99)/2.0
C---------------
	
       do 336 ik = 1,100
          p70inv(ik)=p70inv(ik)*pfct/p70inv(1)
          write(13,1010)h70(ik),p70inv(ik),ozo70(ik)*con/conv,t70inv(ik)
          write(6,1010)h70(ik),p70inv(ik),ozo70(ik)*con/conv,t70inv(ik)
 336   continue    
 1000 FORMAT(I4,5E13.5)
 1010 format(f10.1,1p,e12.5,2e10.3)
C
 9005 continue
      close(13)
      return
 
      END
      
      SUBROUTINE SPLINE(X,Y,N,YP1,YPN,Y2)
C     AUTHOR:  C SEFTOR
C     DATE:    3 FEB 1991
C     PURPOSE: GIVEN ARRAYS X AND Y OF LENGTH N CONTAINING A TABULATED
C      FUNCTION, AND GIVEN VALUES YP1 AND YPN FOR THE FIRST DERIVATIVE OF
C      THE INTERPOLATING FUNCTION AT POINTS 1 AND N, RESPECTIVELY, THIS
C      ROUTINERETURNS AN ARRAY Y2 OF LENGTH N WHICH CONTAINS THE SECOND
C      DERIVATIVES OF THE INTERPOLATING FUNCTION AT THE TABULATED VALUE
C      POINTS X.  IF YP1 AND/OR YPN ARE EQUAL TO 1 X 10**30 OR LARGER,
C      ROUTINE IS SIGNALLED TO SET THE CORRESPONDING BOUNDARY CONDITION FOR
C      A NATURAL SPLINE, WITH ZERO SECOND DERIVATIVE ON THE BOUNDARY.
C      (ALGORITHM TAKEN FROM NUMERICAL RECIPES (FORTRAN), PRESS ET. AL.)
C
      IMPLICIT INTEGER*4(I-N), REAL*8(A-H,O-Z)
      PARAMETER (NMAX=487)
      REAL*8 X(N), Y(N), Y2(N), U(NMAX)
      REAL*8 YP1, YPN
      INTEGER*4 N
      IF (YP1 .GT. .99D30) THEN
         Y2(1) = 0.
         U(1)  = 0.
      ELSE
         Y2(1) = -0.5
         U(1)  = (3./(X(2)-X(1))) * ((Y(2)-Y(1))/(X(2)-X(1)) - YP1)
      ENDIF
      DO 11 I = 2,N-1
         SIG = (X(I)-X(I-1))/(X(I+1)-X(I-1))
         P = SIG * Y2(I-1) + 2.
         Y2(I) = (SIG-1.)/P
         U(I) = (6.*((Y(I+1)-Y(I))/(X(I+1)-X(I)) - (Y(I)-Y(I-1))
     1    /(X(I)-X(I-1)))/(X(I+1)-X(I-1)) - SIG*U(I-1)) / P
11    CONTINUE
      IF (YPN .GT. .99D30) THEN
         QN = 0.
         UN = 0.
      ELSE
         QN = 0.5
         UN = (3./(X(N)-X(N-1))) * (YPN-(Y(N)-Y(N-1))/(X(N)-X(N-1)))
      ENDIF
      Y2(N) = (UN - QN*U(N-1))/(QN*Y2(N-1)+1.)
      DO 12 K = N-1,1,-1
         Y2(K) = Y2(K) * Y2(K+1) + U(K)
12    CONTINUE
      RETURN
      END
      SUBROUTINE SPLINT(XA,YA,Y2A,N,X,Y)
C     AUTHOR:  C. SEFTOR
C     DATE:    3 FEB 1991
C     PURPOSE: GIVEN THE ARRAYS XA AND YA OF LENGTH N, WHICH TABULATE A
C      FUNCTION (WITH THE XA'S IN ORDER), AND GIVEN THE ARRAY Y2A,
C      WHICH IS THE OUTPUT FROM THE SUBROUTINE SPLINE, AND GIVEN A VALUE
C      OF X, THIS ROUTINE RETURNS A CUBIC-SPLINE INTERPOLATED VALUE Y.
C      (ALGORITHM TAKEN FROM NUMERICAL RECIPES (FORTRAN) BY PRESS ET. AL.)
C
      IMPLICIT INTEGER*4(I-N), REAL*8(A-H,O-Z)
      REAL*8 XA(N), YA(N), Y2A(N), X, Y
      REAL*8 A, B
      INTEGER*4 N
      KLO = 1
      KHI = N
10    CONTINUE
      IF (KHI-KLO .GT. 1) THEN
         K = (KHI+KLO)/2
         IF (XA(K) .GT. X) THEN
            KHI = K
         ELSE
            KLO = K
         ENDIF
         GOTO 10
      ENDIF
      H = XA(KHI) - XA(KLO)
      IF (H .EQ. 0.) THEN
         WRITE (23,*)' BAD XA INPUT'
         STOP
      ENDIF
      A = (XA(KHI)-X)/H
      B = (X-XA(KLO))/H
      Y = A*YA(KLO) + B*YA(KHI) +
     1 ((A**3-A)*Y2A(KLO) + (B**3-B)*Y2A(KHI)) * (H**2)/6.
      RETURN
      END

